"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6247],{93336:function(a){function b(a){var b,c;(b=a).languages.diff={coord:[/^(?:\*{3}|-{3}|\+{3}).*$/m,/^@@.*@@$/m,/^\d.*$/m]},Object.keys(c={"deleted-sign":"-","deleted-arrow":"<","inserted-sign":"+","inserted-arrow":">",unchanged:" ",diff:"!"}).forEach(function(a){var d=c[a],e=[];/^\w+$/.test(a)||e.push(/\w+/.exec(a)[0]),"diff"===a&&e.push("bold"),b.languages.diff[a]={pattern:RegExp("^(?:["+d+"].*(?:\r\n?|\n|(?![\\s\\S])))+","m"),alias:e,inside:{line:{pattern:/(.)(?=[\s\S]).*(?:\r\n?|\n)?/,lookbehind:!0},prefix:{pattern:/[\s\S]/,alias:/\w+/.exec(a)[0]}}}}),Object.defineProperty(b.languages.diff,"PREFIXES",{value:c})}a.exports=b,b.displayName="diff",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_diff.a2ad866800f81367.js.map