"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3971],{5199:function(a){function b(a){a.languages.actionscript=a.languages.extend("javascript",{keyword:/\b(?:as|break|case|catch|class|const|default|delete|do|dynamic|each|else|extends|final|finally|for|function|get|if|implements|import|in|include|instanceof|interface|internal|is|namespace|native|new|null|override|package|private|protected|public|return|set|static|super|switch|this|throw|try|typeof|use|var|void|while|with)\b/,operator:/\+\+|--|(?:[+\-*\/%^]|&&?|\|\|?|<<?|>>?>?|[!=]=?)=?|[~?@]/}),a.languages.actionscript["class-name"].alias="function",delete a.languages.actionscript.parameter,delete a.languages.actionscript["literal-property"],a.languages.markup&&a.languages.insertBefore("actionscript","string",{xml:{pattern:/(^|[^.])<\/?\w+(?:\s+[^\s>\/=]+=("|')(?:\\[\s\S]|(?!\2)[^\\])*\2)*\s*\/?>/,lookbehind:!0,inside:a.languages.markup}})}a.exports=b,b.displayName="actionscript",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_actionscript.0064557d15e89230.js.map