(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[9996],{99092:function(a,b,c){(window.__NEXT_P=window.__NEXT_P||[]).push(["/[type]/[entity]",function(){return c(21228)}])},21228:function(a,b,c){"use strict";c.r(b);var d=c(85893),e=c(11163);c(67294);var f=c(4386),g=c(46011),h=function(){var a=(0,e.useRouter)(),b=a.query,c=b.type,h=b.entity,i=a.query,j=i.variant,k=new f.ZP;return(0,d.jsx)(g.Z,{api:k,type:c,entity:h,queryVariant:j})};b.default=h}},function(a){a.O(0,[9774,2888,179],function(){var b;return a(a.s=99092)}),_N_E=a.O()}])
//# sourceMappingURL=[entity]-9e34f7181eeb8a97.js.map