"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[8497],{15958:function(a){function b(a){a.languages.bnf={string:{pattern:/"[^\r\n"]*"|'[^\r\n']*'/},definition:{pattern:/<[^<>\r\n\t]+>(?=\s*::=)/,alias:["rule","keyword"],inside:{punctuation:/^<|>$/}},rule:{pattern:/<[^<>\r\n\t]+>/,inside:{punctuation:/^<|>$/}},operator:/::=|[|()[\]{}*+?]|\.{3}/},a.languages.rbnf=a.languages.bnf}a.exports=b,b.displayName="bnf",b.aliases=["rbnf"]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_bnf.7541e7fe6f3dbc6f.js.map