(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[9635],{62257:function(a,b,c){(window.__NEXT_P=window.__NEXT_P||[]).push(["/admin/users",function(){return c(4945)}])},4945:function(a,b,c){"use strict";c.r(b);var d=c(85893);c(67294);var e=c(57225),f=function(){return(0,d.jsx)(d.Fragment,{children:(0,d.jsx)(e.Z,{})})};b.default=f},57225:function(a,b,c){"use strict";c.d(b,{Z:function(){return s}});var d=c(47568),e=c(34051),f=c.n(e),g=c(85893),h=c(72882),i=c(90629),j=c(7906),k=c(53184),l=c(53816),m=c(53252),n=c(295),o=c(83321),p=c(11163),q=c(67294),r=c(46291);function s(a){var b=a.roleNameProp,c=void 0===b?"":b,e=(0,p.useRouter)(),s=(0,q.useState)([]),t=s[0],u=s[1],v=(0,r.rt)(),w=function(){var a=arguments.length>0&& void 0!==arguments[0]?arguments[0]:"";a&&e.push("/admin/user/"+a)};return(0,q.useEffect)(function(){var a;(a=(0,d.Z)(f().mark(function a(){var b;return f().wrap(function(a){for(;;)switch(a.prev=a.next){case 0:if(b=[],!c){a.next=7;break}return a.next=4,v.getRoleUsers(c);case 4:b=a.sent,a.next=10;break;case 7:return a.next=9,v.getUsers();case 9:b=a.sent;case 10:b&&u(b);case 11:case"end":return a.stop()}},a)})),function(){return a.apply(this,arguments)})()},[]),(0,g.jsxs)(g.Fragment,{children:[(0,g.jsx)("strong",{style:{float:"left",paddingBottom:"1em"},children:"Users"}),(0,g.jsx)(h.Z,{component:i.Z,children:(0,g.jsxs)(j.Z,{sx:{minWidth:300},"aria-label":"Admin User Table",children:[(0,g.jsx)(k.Z,{children:(0,g.jsx)(l.Z,{children:(0,g.jsx)(m.Z,{children:"Name"})})}),(0,g.jsx)(n.Z,{children:null==t?void 0:t.map(function(a,b){return(0,g.jsx)(l.Z,{hover:!0,children:(0,g.jsx)(m.Z,{children:(0,g.jsx)(o.Z,{onClick:function(){return w(a)},children:a})})},b)})})]})})]})}}},function(a){a.O(0,[9774,2888,179],function(){var b;return a(a.s=62257)}),_N_E=a.O()}])
//# sourceMappingURL=users-86ec60a77e865a65.js.map