"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6818],{77935:function(a){function b(a){var b,c,d,e,f;b=a,c=/\\\((?:[^()]|\([^()]*\))*\)/.source,d=RegExp(/(^|[^\\])"(?:[^"\r\n\\]|\\[^\r\n(]|__)*"/.source.replace(/__/g,function(){return c})),e={interpolation:{pattern:RegExp(/((?:^|[^\\])(?:\\{2})*)/.source+c),lookbehind:!0,inside:{content:{pattern:/^(\\\()[\s\S]+(?=\)$)/,lookbehind:!0,inside:null},punctuation:/^\\\(|\)$/}}},f=b.languages.jq={comment:/#.*/,property:{pattern:RegExp(d.source+/(?=\s*:(?!:))/.source),lookbehind:!0,greedy:!0,inside:e},string:{pattern:d,lookbehind:!0,greedy:!0,inside:e},function:{pattern:/(\bdef\s+)[a-z_]\w+/i,lookbehind:!0},variable:/\B\$\w+/,"property-literal":{pattern:/\b[a-z_]\w*(?=\s*:(?!:))/i,alias:"property"},keyword:/\b(?:as|break|catch|def|elif|else|end|foreach|if|import|include|label|module|modulemeta|null|reduce|then|try|while)\b/,boolean:/\b(?:false|true)\b/,number:/(?:\b\d+\.|\B\.)?\b\d+(?:[eE][+-]?\d+)?\b/,operator:[{pattern:/\|=?/,alias:"pipe"},/\.\.|[!=<>]?=|\?\/\/|\/\/=?|[-+*/%]=?|[<>?]|\b(?:and|not|or)\b/],"c-style-function":{pattern:/\b[a-z_]\w*(?=\s*\()/i,alias:"function"},punctuation:/::|[()\[\]{},:;]|\.(?=\s*[\[\w$])/,dot:{pattern:/\./,alias:"important"}},e.interpolation.inside.content.inside=f}a.exports=b,b.displayName="jq",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_jq.5fc2c57f003c8222.js.map