"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[9674],{12208:function(a){function b(a){a.languages.gcode={comment:/;.*|\B\(.*?\)\B/,string:{pattern:/"(?:""|[^"])*"/,greedy:!0},keyword:/\b[GM]\d+(?:\.\d+)?\b/,property:/\b[A-Z]/,checksum:{pattern:/(\*)\d+/,lookbehind:!0,alias:"number"},punctuation:/[:*]/}}a.exports=b,b.displayName="gcode",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_gcode.b7b257f3e9df2832.js.map