"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[1151],{30296:function(a){function b(a){a.languages.editorconfig={comment:/[;#].*/,section:{pattern:/(^[ \t]*)\[.+\]/m,lookbehind:!0,alias:"selector",inside:{regex:/\\\\[\[\]{},!?.*]/,operator:/[!?]|\.\.|\*{1,2}/,punctuation:/[\[\]{},]/}},key:{pattern:/(^[ \t]*)[^\s=]+(?=[ \t]*=)/m,lookbehind:!0,alias:"attr-name"},value:{pattern:/=.*/,alias:"attr-value",inside:{punctuation:/^=/}}}}a.exports=b,b.displayName="editorconfig",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_editorconfig.ddfb1264fc373bf7.js.map