"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[4732],{42876:function(a){function b(a){var b,c,d;b=a,d={"equation-command":{pattern:c=/\\(?:[^a-z()[\]]|[a-z*]+)/i,alias:"regex"}},b.languages.latex={comment:/%.*/,cdata:{pattern:/(\\begin\{((?:lstlisting|verbatim)\*?)\})[\s\S]*?(?=\\end\{\2\})/,lookbehind:!0},equation:[{pattern:/\$\$(?:\\[\s\S]|[^\\$])+\$\$|\$(?:\\[\s\S]|[^\\$])+\$|\\\([\s\S]*?\\\)|\\\[[\s\S]*?\\\]/,inside:d,alias:"string"},{pattern:/(\\begin\{((?:align|eqnarray|equation|gather|math|multline)\*?)\})[\s\S]*?(?=\\end\{\2\})/,lookbehind:!0,inside:d,alias:"string"}],keyword:{pattern:/(\\(?:begin|cite|documentclass|end|label|ref|usepackage)(?:\[[^\]]+\])?\{)[^}]+(?=\})/,lookbehind:!0},url:{pattern:/(\\url\{)[^}]+(?=\})/,lookbehind:!0},headline:{pattern:/(\\(?:chapter|frametitle|paragraph|part|section|subparagraph|subsection|subsubparagraph|subsubsection|subsubsubparagraph)\*?(?:\[[^\]]+\])?\{)[^}]+(?=\})/,lookbehind:!0,alias:"class-name"},function:{pattern:c,alias:"selector"},punctuation:/[[\]{}&]/},b.languages.tex=b.languages.latex,b.languages.context=b.languages.latex}a.exports=b,b.displayName="latex",b.aliases=["tex","context"]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_latex.8ee2e02a70dd3d10.js.map