"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[8825,902,9461],{9858:function(a){function b(a){var b,c;Object.defineProperty(c=(b=a).languages.javadoclike={parameter:{pattern:/(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*@(?:arg|arguments|param)\s+)\w+/m,lookbehind:!0},keyword:{pattern:/(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*|\{)@[a-z][a-zA-Z-]+\b/m,lookbehind:!0},punctuation:/[{}]/},"addSupport",{value:function(a,c){"string"==typeof a&&(a=[a]),a.forEach(function(a){!function(a,c){var d="doc-comment",e=b.languages[a];if(e){var f=e[d];if(!f){var g={};g[d]={pattern:/(^|[^\\])\/\*\*[^/][\s\S]*?(?:\*\/|$)/,lookbehind:!0,alias:"comment"},f=(e=b.languages.insertBefore(a,"comment",g))[d]}if(f instanceof RegExp&&(f=e[d]={pattern:f}),Array.isArray(f))for(var h=0,i=f.length;h<i;h++)f[h]instanceof RegExp&&(f[h]={pattern:f[h]}),c(f[h]);else c(f)}}(a,function(a){a.inside||(a.inside={}),a.inside.rest=c})})}}),c.addSupport(["java","javascript","php"],c)}a.exports=b,b.displayName="javadoclike",b.aliases=[]},25319:function(a,b,c){var d=c(9858),e=c(4979);function f(a){var b,c,f,g;a.register(d),a.register(e),c=(b=a).languages.javascript,g="(@(?:arg|argument|param|property)\\s+(?:"+(f=/\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})+\}/.source)+"\\s+)?)",b.languages.jsdoc=b.languages.extend("javadoclike",{parameter:{pattern:RegExp(g+/(?:(?!\s)[$\w\xA0-\uFFFF.])+(?=\s|$)/.source),lookbehind:!0,inside:{punctuation:/\./}}}),b.languages.insertBefore("jsdoc","keyword",{"optional-parameter":{pattern:RegExp(g+/\[(?:(?!\s)[$\w\xA0-\uFFFF.])+(?:=[^[\]]+)?\](?=\s|$)/.source),lookbehind:!0,inside:{parameter:{pattern:/(^\[)[$\w\xA0-\uFFFF\.]+/,lookbehind:!0,inside:{punctuation:/\./}},code:{pattern:/(=)[\s\S]*(?=\]$)/,lookbehind:!0,inside:c,alias:"language-javascript"},punctuation:/[=[\]]/}},"class-name":[{pattern:RegExp(/(@(?:augments|class|extends|interface|memberof!?|template|this|typedef)\s+(?:<TYPE>\s+)?)[A-Z]\w*(?:\.[A-Z]\w*)*/.source.replace(/<TYPE>/g,function(){return f})),lookbehind:!0,inside:{punctuation:/\./}},{pattern:RegExp("(@[a-z]+\\s+)"+f),lookbehind:!0,inside:{string:c.string,number:c.number,boolean:c.boolean,keyword:b.languages.typescript.keyword,operator:/=>|\.\.\.|[&|?:*]/,punctuation:/[.,;=<>{}()[\]]/}}],example:{pattern:/(@example\s+(?!\s))(?:[^@\s]|\s+(?!\s))+?(?=\s*(?:\*\s*)?(?:@\w|\*\/))/,lookbehind:!0,inside:{code:{pattern:/^([\t ]*(?:\*\s*)?)\S.*$/m,lookbehind:!0,inside:c,alias:"language-javascript"}}}}),b.languages.javadoclike.addSupport("javascript",b.languages.jsdoc)}a.exports=f,f.displayName="jsdoc",f.aliases=[]},4979:function(a){function b(a){var b,c;(b=a).languages.typescript=b.languages.extend("javascript",{"class-name":{pattern:/(\b(?:class|extends|implements|instanceof|interface|new|type)\s+)(?!keyof\b)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?:\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>)?/,lookbehind:!0,greedy:!0,inside:null},builtin:/\b(?:Array|Function|Promise|any|boolean|console|never|number|string|symbol|unknown)\b/}),b.languages.typescript.keyword.push(/\b(?:abstract|declare|is|keyof|readonly|require)\b/,/\b(?:asserts|infer|interface|module|namespace|type)\b(?=\s*(?:[{_$a-zA-Z\xA0-\uFFFF]|$))/,/\btype\b(?=\s*(?:[\{*]|$))/),delete b.languages.typescript.parameter,delete b.languages.typescript["literal-property"],delete(c=b.languages.extend("typescript",{}))["class-name"],b.languages.typescript["class-name"].inside=c,b.languages.insertBefore("typescript","function",{decorator:{pattern:/@[$\w\xA0-\uFFFF]+/,inside:{at:{pattern:/^@/,alias:"operator"},function:/^[\s\S]+/}},"generic-function":{pattern:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>(?=\s*\()/,greedy:!0,inside:{function:/^#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*/,generic:{pattern:/<[\s\S]+/,alias:"class-name",inside:c}}}}),b.languages.ts=b.languages.typescript}a.exports=b,b.displayName="typescript",b.aliases=["ts"]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_jsdoc.c7377e771b995b53.js.map