(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[9603],{92774:function(a,b,c){(window.__NEXT_P=window.__NEXT_P||[]).push(["/search",function(){return c(99505)}])},99505:function(a,b,c){"use strict";c.r(b);var d=c(88277);b.default=d.Z}},function(a){a.O(0,[9774,2888,179],function(){var b;return a(a.s=92774)}),_N_E=a.O()}])
//# sourceMappingURL=search-31b270dcde7e866f.js.map