"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[7561],{80963:function(a,b,c){var d=c(45950);function e(a){a.register(d),a.languages.jsonp=a.languages.extend("json",{punctuation:/[{}[\]();,.]/}),a.languages.insertBefore("jsonp","punctuation",{function:/(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*\()/})}a.exports=e,e.displayName="jsonp",e.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_jsonp.26f63a803f39fd73.js.map