self.__BUILD_MANIFEST=function(a){return{__rewrites:{beforeFiles:[],afterFiles:[],fallback:[]},"/":["static/chunks/pages/index-f45534c2dfd80b9f.js"],"/404":["static/chunks/pages/404-6ae1c67f55552cb1.js"],"/_error":["static/chunks/pages/_error-b04df28c0915943c.js"],"/admin/group/[groupName]":["static/chunks/pages/admin/group/[groupName]-d236f7ada869147f.js"],"/admin/groups":["static/chunks/pages/admin/groups-4d0079606d867a62.js"],"/admin/role/[roleName]":[a,"static/chunks/pages/admin/role/[roleName]-1ca31d5eb3b6d7b1.js"],"/admin/roles":[a,"static/chunks/pages/admin/roles-54866a847a0a981b.js"],"/admin/user/[userName]":[a,"static/chunks/pages/admin/user/[userName]-80cb5fc864a80c88.js"],"/admin/users":["static/chunks/pages/admin/users-86ec60a77e865a65.js"],"/query":["static/chunks/pages/query-469b79090675947c.js"],"/search":["static/chunks/pages/search-31b270dcde7e866f.js"],"/tasks":["static/chunks/pages/tasks-c704d59f025a5d88.js"],"/[type]":["static/chunks/pages/[type]-9400f226b78bb25f.js"],"/[type]/[entity]":["static/chunks/pages/[type]/[entity]-9e34f7181eeb8a97.js"],sortedPages:["/","/404","/_app","/_error","/admin/group/[groupName]","/admin/groups","/admin/role/[roleName]","/admin/roles","/admin/user/[userName]","/admin/users","/query","/search","/tasks","/[type]","/[type]/[entity]"]}}("static/chunks/9562-4a53dde669a90a49.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB()