"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3279],{2329:function(a){function b(a){!function(a){function b(a,b,c){return{pattern:RegExp("<#"+a+"[\\s\\S]*?#>"),alias:"block",inside:{delimiter:{pattern:RegExp("^<#"+a+"|#>$"),alias:"important"},content:{pattern:/[\s\S]+/,inside:b,alias:c}}}}a.languages["t4-templating"]=Object.defineProperty({},"createT4",{value:function(c){var d=a.languages[c],e="language-"+c;return{block:{pattern:/<#[\s\S]+?#>/,inside:{directive:b("@",{"attr-value":{pattern:/=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">=]+)/,inside:{punctuation:/^=|^["']|["']$/}},keyword:/\b\w+(?=\s)/,"attr-name":/\b\w+/}),expression:b("=",d,e),"class-feature":b("\\+",d,e),standard:b("",d,e)}}}}})}(a)}a.exports=b,b.displayName="t4Templating",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_t4Templating.f2df67b08d696466.js.map