"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[982],{20349:function(a){function b(a){var b,c,d;(b=a).languages.xquery=b.languages.extend("markup",{"xquery-comment":{pattern:/\(:[\s\S]*?:\)/,greedy:!0,alias:"comment"},string:{pattern:/(["'])(?:\1\1|(?!\1)[\s\S])*\1/,greedy:!0},extension:{pattern:/\(#.+?#\)/,alias:"symbol"},variable:/\$[-\w:]+/,axis:{pattern:/(^|[^-])(?:ancestor(?:-or-self)?|attribute|child|descendant(?:-or-self)?|following(?:-sibling)?|parent|preceding(?:-sibling)?|self)(?=::)/,lookbehind:!0,alias:"operator"},"keyword-operator":{pattern:/(^|[^:-])\b(?:and|castable as|div|eq|except|ge|gt|idiv|instance of|intersect|is|le|lt|mod|ne|or|union)\b(?=$|[^:-])/,lookbehind:!0,alias:"operator"},keyword:{pattern:/(^|[^:-])\b(?:as|ascending|at|base-uri|boundary-space|case|cast as|collation|construction|copy-namespaces|declare|default|descending|else|empty (?:greatest|least)|encoding|every|external|for|function|if|import|in|inherit|lax|let|map|module|namespace|no-inherit|no-preserve|option|order(?: by|ed|ing)?|preserve|return|satisfies|schema|some|stable|strict|strip|then|to|treat as|typeswitch|unordered|validate|variable|version|where|xquery)\b(?=$|[^:-])/,lookbehind:!0},function:/[\w-]+(?::[\w-]+)*(?=\s*\()/,"xquery-element":{pattern:/(element\s+)[\w-]+(?::[\w-]+)*/,lookbehind:!0,alias:"tag"},"xquery-attribute":{pattern:/(attribute\s+)[\w-]+(?::[\w-]+)*/,lookbehind:!0,alias:"attr-name"},builtin:{pattern:/(^|[^:-])\b(?:attribute|comment|document|element|processing-instruction|text|xs:(?:ENTITIES|ENTITY|ID|IDREFS?|NCName|NMTOKENS?|NOTATION|Name|QName|anyAtomicType|anyType|anyURI|base64Binary|boolean|byte|date|dateTime|dayTimeDuration|decimal|double|duration|float|gDay|gMonth|gMonthDay|gYear|gYearMonth|hexBinary|int|integer|language|long|negativeInteger|nonNegativeInteger|nonPositiveInteger|normalizedString|positiveInteger|short|string|time|token|unsigned(?:Byte|Int|Long|Short)|untyped(?:Atomic)?|yearMonthDuration))\b(?=$|[^:-])/,lookbehind:!0},number:/\b\d+(?:\.\d+)?(?:E[+-]?\d+)?/,operator:[/[+*=?|@]|\.\.?|:=|!=|<[=<]?|>[=>]?/,{pattern:/(\s)-(?=\s)/,lookbehind:!0}],punctuation:/[[\](){},;:/]/}),b.languages.xquery.tag.pattern=/<\/?(?!\d)[^\s>\/=$<%]+(?:\s+[^\s>\/=]+(?:=(?:("|')(?:\\[\s\S]|\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}|(?!\1)[^\\])*\1|[^\s'">=]+))?)*\s*\/?>/,b.languages.xquery.tag.inside["attr-value"].pattern=/=(?:("|')(?:\\[\s\S]|\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}|(?!\1)[^\\])*\1|[^\s'">=]+)/,b.languages.xquery.tag.inside["attr-value"].inside.punctuation=/^="|"$/,b.languages.xquery.tag.inside["attr-value"].inside.expression={pattern:/\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}/,inside:b.languages.xquery,alias:"language-xquery"},c=function(a){return"string"==typeof a?a:"string"==typeof a.content?a.content:a.content.map(c).join("")},d=function(a){for(var e=[],f=0;f<a.length;f++){var g=a[f],h=!1;if("string"!=typeof g&&("tag"===g.type&&g.content[0]&&"tag"===g.content[0].type?"</"===g.content[0].content[0].content?e.length>0&&e[e.length-1].tagName===c(g.content[0].content[1])&&e.pop():"/>"===g.content[g.content.length-1].content||e.push({tagName:c(g.content[0].content[1]),openedBraces:0}):!(e.length>0)||"punctuation"!==g.type||"{"!==g.content||a[f+1]&&"punctuation"===a[f+1].type&&"{"===a[f+1].content||a[f-1]&&"plain-text"===a[f-1].type&&"{"===a[f-1].content?e.length>0&&e[e.length-1].openedBraces>0&&"punctuation"===g.type&&"}"===g.content?e[e.length-1].openedBraces--:"comment"!==g.type&&(h=!0):e[e.length-1].openedBraces++),(h||"string"==typeof g)&&e.length>0&&0===e[e.length-1].openedBraces){var i=c(g);f<a.length-1&&("string"==typeof a[f+1]||"plain-text"===a[f+1].type)&&(i+=c(a[f+1]),a.splice(f+1,1)),f>0&&("string"==typeof a[f-1]||"plain-text"===a[f-1].type)&&(i=c(a[f-1])+i,a.splice(f-1,1),f--),/^\s+$/.test(i)?a[f]=i:a[f]=new b.Token("plain-text",i,null,i)}g.content&&"string"!=typeof g.content&&d(g.content)}},b.hooks.add("after-tokenize",function(a){"xquery"===a.language&&d(a.tokens)})}a.exports=b,b.displayName="xquery",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_xquery.36c703ae699b7e8b.js.map